from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.linked_term_custom_data import LinkedTermCustomData


class LinkedTermConfigurationRequest(BaseModel):
    external_term_id: Optional[str] = None
    term_name: Optional[str] = None
    description: Optional[str] = None
    external_product_ids: Optional[str] = None
    subscription_management_url: Optional[str] = None
    custom_data: Optional['LinkedTermCustomData'] = None


LinkedTermConfigurationRequest.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Tooltip(BaseModel):
    enabled: Optional[bool] = None
    type: Optional[str] = None
    text: Optional[str] = None


Tooltip.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ReportResponse(BaseModel):
    error: Optional[str] = None
    message: Optional[str] = None
    report: Optional[str] = None


ReportResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.advanced_options import AdvancedOptions
from pianosdk.publisher.models.light_offer import LightOffer
from typing import List


class TermChangeOption(BaseModel):
    term_change_option_id: Optional[str] = None
    from_term_id: Optional[str] = None
    from_term_name: Optional[str] = None
    from_period_id: Optional[str] = None
    from_period_name: Optional[str] = None
    from_resource_id: Optional[str] = None
    from_resource_name: Optional[str] = None
    from_billing_plan: Optional[str] = None
    to_term_id: Optional[str] = None
    to_term_name: Optional[str] = None
    to_period_id: Optional[str] = None
    to_period_name: Optional[str] = None
    to_resource_id: Optional[str] = None
    to_resource_name: Optional[str] = None
    to_billing_plan: Optional[str] = None
    billing_timing: Optional[str] = None
    immediate_access: Optional[bool] = None
    prorate_access: Optional[bool] = None
    description: Optional[str] = None
    include_trial: Optional[bool] = None
    to_scheduled: Optional[bool] = None
    from_scheduled: Optional[bool] = None
    shared_account_count: Optional[int] = None
    collect_address: Optional[bool] = None
    upgrade_offers: Optional['List[LightOffer]'] = None
    advanced_options: Optional['AdvancedOptions'] = None


TermChangeOption.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.upgrade_options_variant import UpgradeOptionsVariant
from typing import List


class UpgradeOptionVariants(BaseModel):
    eligible_options: Optional['List[UpgradeOptionsVariant]'] = None
    ineligible_options: Optional['List[UpgradeOptionsVariant]'] = None


UpgradeOptionVariants.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Notification(BaseModel):
    text: Optional[str] = None
    create_date: Optional[datetime] = None
    type: Optional[str] = None
    object_type: Optional[str] = None
    object_id: Optional[str] = None
    initiator: Optional[str] = None


Notification.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LocaleModel(BaseModel):
    locale: Optional[str] = None
    label: Optional[str] = None
    localized_label: Optional[str] = None
    is_default: Optional[bool] = None
    is_enabled: Optional[bool] = None
    is_rtl: Optional[bool] = None


LocaleModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Mail(BaseModel):
    email_subject: Optional[str] = None
    email_body: Optional[str] = None


Mail.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class DatatransInitiateTransactionResponse(BaseModel):
    transaction_id: Optional[str] = None


DatatransInitiateTransactionResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.anon.models.resource_dto import ResourceDto
from pianosdk.anon.models.user_dto import UserDto


class AccessDTO(BaseModel):
    access_id: Optional[str] = None
    parent_access_id: Optional[str] = None
    granted: Optional[bool] = None
    user: Optional['UserDto'] = None
    resource: Optional['ResourceDto'] = None
    expire_date: Optional[datetime] = None
    start_date: Optional[datetime] = None
    can_revoke_access: Optional[bool] = None
    custom_data: Optional[str] = None


AccessDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PublicCaptchaConfig(BaseModel):
    aid: Optional[str] = None
    enabled: Optional[bool] = None
    captcha3_site_key: Optional[str] = None


PublicCaptchaConfig.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user import User


class InquiryComment(BaseModel):
    comment_id: Optional[str] = None
    submitter_type: Optional[int] = None
    create_date: Optional[str] = None
    message: Optional[str] = None
    user: Optional['User'] = None
    email: Optional[str] = None
    name: Optional[str] = None
    personal_name: Optional[str] = None
    internal: Optional[str] = None


InquiryComment.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.term_short import TermShort


class ChangeSubscriptionTermStep(BaseModel):
    from_term: Optional['TermShort'] = None
    to_term: Optional['TermShort'] = None
    optional: Optional[bool] = None
    enabled: Optional[bool] = None


ChangeSubscriptionTermStep.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.delivery_schedule_constraint_violation_dto import DeliveryScheduleConstraintViolationDTO
from typing import List


class DeliveryScheduleConstraintViolationsDTO(BaseModel):
    constraint_violations: Optional['List[DeliveryScheduleConstraintViolationDTO]'] = None


DeliveryScheduleConstraintViolationsDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import Any
from typing import Dict
from typing import List


class Validator(BaseModel):
    type: Optional[str] = None
    params: Optional['Dict[str, Any]'] = None
    error_message: Optional[str] = None


Validator.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class BrowserSegment(BaseModel):
    segment_id: Optional[str] = None
    name: Optional[str] = None
    sort_order: Optional[int] = None
    filter: Optional[str] = None


BrowserSegment.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PromotionFixedDiscount(BaseModel):
    fixed_discount_id: Optional[str] = None
    currency: Optional[str] = None
    amount: Optional[str] = None
    amount_value: Optional[float] = None


PromotionFixedDiscount.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ResourceCompactStats(BaseModel):
    rid: Optional[str] = None
    has_conversions: Optional[bool] = None
    num_terms: Optional[int] = None
    num_bundles: Optional[int] = None
    tags: Optional[str] = None


ResourceCompactStats.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class EraseUserPaymentInfo(BaseModel):
    user_payment_info_id: Optional[str] = None
    billing_zip_code: Optional[str] = None
    residence_country: Optional[str] = None
    provider_fields: Optional[str] = None
    upi_nickname: Optional[str] = None
    upi_postal_code: Optional[str] = None
    funding_source: Optional[str] = None
    pin_code: Optional[str] = None
    account_number: Optional[str] = None
    external_transaction_id: Optional[str] = None
    upi_ext_customer_id: Optional[str] = None


EraseUserPaymentInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class C2ExperienceConfig(BaseModel):
    aid: Optional[str] = None
    version: Optional[int] = None
    js: Optional[str] = None
    published: Optional[bool] = None
    published_date: Optional[datetime] = None


C2ExperienceConfig.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.reallocation_batch import ReallocationBatch
from typing import List


class TermPeriodReallocation(BaseModel):
    term_pub_id: Optional[str] = None
    name: Optional[str] = None
    period_pub_id: Optional[str] = None
    next_period_sell_date: Optional[datetime] = None
    access_end_date: Optional[datetime] = None
    renewal_date: Optional[datetime] = None
    total_subscribers: Optional[int] = None
    subscribers_batches: Optional['List[ReallocationBatch]'] = None
    batches_with_pending_changes: Optional['List[ReallocationBatch]'] = None
    is_reallocation_in_progress: Optional[bool] = None


TermPeriodReallocation.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.currency import Currency
from typing import List


class DatatransProperties(BaseModel):
    currencies: Optional['List[Currency]'] = None
    collect_cardholder_name: Optional[bool] = None


DatatransProperties.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.term_from import TermFrom
from pianosdk.publisher.models.term_to_eligible import TermToEligible
from pianosdk.publisher.models.term_to_ineligible import TermToIneligible
from pianosdk.publisher.models.term_to_upgrade_option import TermToUpgradeOption
from typing import List


class SubscriptionTermChangeOptions(BaseModel):
    change_from: Optional['TermFrom'] = None
    change_options: Optional['List[TermToUpgradeOption]'] = None
    eligible_options: Optional['List[TermToEligible]'] = None
    ineligible_options: Optional['List[TermToIneligible]'] = None


SubscriptionTermChangeOptions.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.import_item import ImportItem
from typing import List


class ImportSection(BaseModel):
    id: Optional[str] = None
    name: Optional[str] = None
    import_list_items: Optional['List[ImportItem]'] = None


ImportSection.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ContinuousRenewalDetails(BaseModel):
    enabled: Optional[bool] = None


ContinuousRenewalDetails.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.offer_template import OfferTemplate
from typing import List


class OfferTemplateCategories(BaseModel):
    category_id: Optional[str] = None
    category_id_localized: Optional[str] = None
    count_templates: Optional[int] = None
    offer_template_models: Optional['List[OfferTemplate]'] = None


OfferTemplateCategories.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class RegionSimpleModel(BaseModel):
    region_name: Optional[str] = None
    pub_id: Optional[str] = None


RegionSimpleModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LinkedTermCustomData(BaseModel):
    value: Optional[str] = None


LinkedTermCustomData.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import Any
from typing import List


class BillingPlanTable(BaseModel):
    payment_billing_plan_table: Optional['List[Any]'] = None


BillingPlanTable.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.show_option_in_channel_stats import ShowOptionInChannelStats
from typing import List


class ShowOptionInCounters(BaseModel):
    selected_channel: Optional[str] = None
    show_option_in_channels: Optional['List[ShowOptionInChannelStats]'] = None


ShowOptionInCounters.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.offer_template import OfferTemplate
from pianosdk.publisher.models.offer_template_categories import OfferTemplateCategories
from pianosdk.publisher.models.offer_template_histories import OfferTemplateHistories
from pianosdk.publisher.models.offer_template_version import OfferTemplateVersion
from pianosdk.publisher.models.template_config import TemplateConfig


class PublisherOfferTemplateApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def archive(self, aid: str, offer_template_id: str, history_comment: str = None) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/offer/template/archive'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id),
            'history_comment': _encode_parameter(history_comment)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def convert_boilerplate_to_offer_template(self, aid: str, offer_template_id: str, name: str, category_id: str, description: str = None) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/convertBoilerplateToTemplate'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id),
            'name': _encode_parameter(name),
            'description': _encode_parameter(description),
            'category_id': _encode_parameter(category_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result

    def count(self, aid: str) -> ApiResponse[int]:
        _url_path = '/api/v3/publisher/offer/template/count'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, int)
        return _result

    def create(self, aid: str, name: str, description: str = None, category_id: str = None, history_comment: str = None) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'name': _encode_parameter(name),
            'description': _encode_parameter(description),
            'category_id': _encode_parameter(category_id),
            'history_comment': _encode_parameter(history_comment)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result

    def delete(self, aid: str, offer_template_id: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/offer/template/delete'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def get(self, aid: str, offer_template_id: str) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/get'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result

    def get_default_code(self, aid: str, offer_template_id: str) -> ApiResponse[TemplateConfig]:
        _url_path = '/api/v3/publisher/offer/template/defaultCode'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, TemplateConfig)
        return _result

    def get_duplicate(self, aid: str, offer_template_id: str, history_comment: str = None, duplicate_variants: bool = None) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/duplicate'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id),
            'history_comment': _encode_parameter(history_comment),
            'duplicate_variants': _encode_parameter(duplicate_variants)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result

    def get_history(self, aid: str, offer_template_id: str) -> ApiResponse[OfferTemplateHistories]:
        _url_path = '/api/v3/publisher/offer/template/getHistory'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateHistories)
        return _result

    def list(self, aid: str, offset: int = 0, limit: int = 100, q: str = None, order_by: str = None, order_direction: str = None, status: str = None, variant_status: str = None, filters: List[str] = None) -> ApiResponse[List[OfferTemplate]]:
        _url_path = '/api/v3/publisher/offer/template/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'status': _encode_parameter(status),
            'variant_status': _encode_parameter(variant_status),
            'filters': _encode_parameter(filters)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplate)
        return _result

    def list_boilerplates(self, aid: str, categories: List[str], offset: int = 0, limit: int = 100, q: str = None, order_by: str = None, order_direction: str = None, status: str = None, template_type: str = None, ensure_templates_exists: bool = None, engine: str = None) -> ApiResponse[List[OfferTemplateCategories]]:
        _url_path = '/api/v3/publisher/offer/template/listBoilerplates'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'status': _encode_parameter(status),
            'templateType': _encode_parameter(template_type),
            'categories': _encode_parameter(categories),
            'ensure_templates_exists': _encode_parameter(ensure_templates_exists),
            'engine': _encode_parameter(engine)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateCategories)
        return _result

    def list_by_category(self, aid: str, categories: List[str], offset: int = 0, limit: int = 100, status: str = None, template_type: str = None, q: str = None, order_by: str = None, order_direction: str = None, ensure_templates_exists: bool = None, engine: str = None) -> ApiResponse[List[OfferTemplateCategories]]:
        _url_path = '/api/v3/publisher/offer/template/listByCategory'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'status': _encode_parameter(status),
            'templateType': _encode_parameter(template_type),
            'categories': _encode_parameter(categories),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'ensure_templates_exists': _encode_parameter(ensure_templates_exists),
            'engine': _encode_parameter(engine)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateCategories)
        return _result

    def make_global(self, aid: str, offer_template_id: str) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/makeGlobal'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result

    def restore(self, aid: str, offer_template_id: str, history_comment: str = None) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/offer/template/restore'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id),
            'history_comment': _encode_parameter(history_comment)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def update(self, aid: str, offer_template_id: str, name: str, description: str = None, category_id: str = None, thumbnail_image_url: str = None, history_comment: str = None) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id),
            'name': _encode_parameter(name),
            'description': _encode_parameter(description),
            'category_id': _encode_parameter(category_id),
            'thumbnail_image_url': _encode_parameter(thumbnail_image_url),
            'history_comment': _encode_parameter(history_comment)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result

    def update_full(self, aid: str, offer_template_id: str, name: str, category_id: str, description: str = None, thumbnail_image_url: str = None, action: str = None, version_name: str = None, version: int = None, content1_type: str = None, content1_value: str = None, content2_type: str = None, content2_value: str = None, content3_type: str = None, content3_value: str = None, external_css_list: str = None, content_field_list: str = None, history_comment: str = None, is_validate: bool = None) -> ApiResponse[OfferTemplateVersion]:
        _url_path = '/api/v3/publisher/offer/template/updatefull'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'offer_template_id': _encode_parameter(offer_template_id),
            'name': _encode_parameter(name),
            'description': _encode_parameter(description),
            'category_id': _encode_parameter(category_id),
            'thumbnail_image_url': _encode_parameter(thumbnail_image_url),
            'action': _encode_parameter(action),
            'version_name': _encode_parameter(version_name),
            'version': _encode_parameter(version),
            'content1_type': _encode_parameter(content1_type),
            'content1_value': _encode_parameter(content1_value),
            'content2_type': _encode_parameter(content2_type),
            'content2_value': _encode_parameter(content2_value),
            'content3_type': _encode_parameter(content3_type),
            'content3_value': _encode_parameter(content3_value),
            'external_css_list': _encode_parameter(external_css_list),
            'content_field_list': _encode_parameter(content_field_list),
            'history_comment': _encode_parameter(history_comment),
            'is_validate': _encode_parameter(is_validate)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, OfferTemplateVersion)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class NewscycleConfiguration(BaseModel):
    url: Optional[str] = None
    api_url: Optional[str] = None
    web_pages_url: Optional[str] = None
    site_id: Optional[str] = None
    synchronization_url: Optional[str] = None


NewscycleConfiguration.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ContractUser(BaseModel):
    email: Optional[str] = None
    last_name: Optional[str] = None
    first_name: Optional[str] = None
    contract_user_id: Optional[str] = None
    status: Optional[str] = None


ContractUser.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class DeploymentApp(BaseModel):
    aid: Optional[str] = None
    name: Optional[str] = None
    app_logo: Optional[str] = None


DeploymentApp.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.address_config import AddressConfig
from typing import List


class ExperianConfiguration(BaseModel):
    experian_token: Optional[str] = None
    pd_address_settings_enabled: Optional[bool] = None
    experian_address_config: Optional['List[AddressConfig]'] = None


ExperianConfiguration.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.anon.models.linked_term_event_page_view_content import LinkedTermEventPageViewContent
from pianosdk.anon.models.linked_term_event_session_offer import LinkedTermEventSessionOffer


class LinkedTermEventSession(BaseModel):
    tracking_id: Optional[str] = None
    tbc: Optional[str] = None
    pcid: Optional[str] = None
    offer: Optional['LinkedTermEventSessionOffer'] = None
    page_view_id: Optional[str] = None
    page_view_content: Optional['LinkedTermEventPageViewContent'] = None
    user_agent: Optional[str] = None
    consents: Optional[str] = None
    previous_user_segments: Optional[str] = None
    user_state: Optional[str] = None


LinkedTermEventSession.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LinkedTermCustomFieldParams(BaseModel):
    name: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None


LinkedTermCustomFieldParams.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.id.models.token_response import TokenResponse


class PublisherTokenApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def refresh_token(self, aid: str, refresh_token: str, authorization: str = None) -> ApiResponse[TokenResponse]:
        _url_path = '/id/api/v1/publisher/token/refresh'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'refresh_token': _encode_parameter(refresh_token)
        }

        _headers = {
            'Authorization': authorization,
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, TokenResponse)
        return _result

    def token_verify_session(self, aid: str, token: str, authorization: str = None) -> ApiResponse[TokenResponse]:
        _url_path = '/id/api/v1/publisher/token/verify'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'token': _encode_parameter(token)
        }

        _headers = {
            'Authorization': authorization,
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, TokenResponse)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LightweightChurnPreventionLogic(BaseModel):
    logic_id: Optional[str] = None
    name: Optional[str] = None
    default: Optional[bool] = None
    max_grace_period: Optional[int] = None


LightweightChurnPreventionLogic.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.experience import Experience
from pianosdk.publisher.models.money import Money
from typing import List


class ExperienceStat(BaseModel):
    experience: Optional['Experience'] = None
    conversion_count: Optional[int] = None
    net_revenues: Optional['List[Money]'] = None


ExperienceStat.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class LinkAllStatusResponse(BaseModel):
    task_id: Optional[str] = None
    status: Optional[str] = None
    fail_message: Optional[str] = None


LinkAllStatusResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class AddressConfig(BaseModel):
    selected: Optional[bool] = None
    selected_by_default: Optional[bool] = None
    address_field: Optional[str] = None
    required: Optional[bool] = None
    default_value: Optional[str] = None
    field: Optional[str] = None


AddressConfig.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.comment_action import CommentAction
from pianosdk.publisher.models.inquiry_comment import InquiryComment
from pianosdk.publisher.models.payment_inquiry import PaymentInquiry


class PublisherInquiryApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def add_comment(self, aid: str, payment_inquiry_id: str, action: List[str], comment: str, internal: bool) -> ApiResponse[PaymentInquiry]:
        _url_path = '/api/v3/publisher/inquiry/add'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'payment_inquiry_id': _encode_parameter(payment_inquiry_id),
            'action': _encode_parameter(action),
            'comment': _encode_parameter(comment),
            'internal': _encode_parameter(internal)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, PaymentInquiry)
        return _result

    def comments(self, aid: str, payment_inquiry_id: str) -> ApiResponse[List[InquiryComment]]:
        _url_path = '/api/v3/publisher/inquiry/comments'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'payment_inquiry_id': _encode_parameter(payment_inquiry_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, InquiryComment)
        return _result

    def details(self, aid: str, payment_inquiry_id: str) -> ApiResponse[PaymentInquiry]:
        _url_path = '/api/v3/publisher/inquiry/details'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'payment_inquiry_id': _encode_parameter(payment_inquiry_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, PaymentInquiry)
        return _result

    def get_inquiry_actions(self, aid: str, payment_inquiry_id: str) -> ApiResponse[List[CommentAction]]:
        _url_path = '/api/v3/publisher/inquiry/inquiryActions'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'payment_inquiry_id': _encode_parameter(payment_inquiry_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, CommentAction)
        return _result

    def last_comment(self, aid: str, uid: str) -> ApiResponse[PaymentInquiry]:
        _url_path = '/api/v3/publisher/inquiry/lastComment'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, PaymentInquiry)
        return _result

    def list(self, aid: str, uid: str, offset: int = 0, limit: int = 100) -> ApiResponse[List[PaymentInquiry]]:
        _url_path = '/api/v3/publisher/inquiry/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, PaymentInquiry)
        return _result

    def unresolved_count(self, aid: str, uid: str = None) -> ApiResponse[int]:
        _url_path = '/api/v3/publisher/inquiry/unresolvedCount'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, int)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class SchedulePeriod(BaseModel):
    period_id: Optional[str] = None
    name: Optional[str] = None
    sell_date: Optional[datetime] = None
    begin_date: Optional[datetime] = None
    end_date: Optional[datetime] = None
    status: Optional[str] = None


SchedulePeriod.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.deployment_content_fields import DeploymentContentFields
from pianosdk.publisher.models.deployment_variant_param_dto import DeploymentVariantParamDto
from pianosdk.publisher.models.user import User
from typing import List


class DeploymentAppDetails(BaseModel):
    item_id: Optional[str] = None
    status: Optional[str] = None
    aid: Optional[str] = None
    name: Optional[str] = None
    app_logo: Optional[str] = None
    offer_template_id: Optional[str] = None
    offer_template_name: Optional[str] = None
    version: Optional[int] = None
    update_date: Optional[datetime] = None
    update_by: Optional['User'] = None
    content_field_list: Optional['List[DeploymentContentFields]'] = None
    variant_list: Optional['List[DeploymentVariantParamDto]'] = None
    selected: Optional[str] = None


DeploymentAppDetails.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.external_api_field import ExternalAPIField
from pianosdk.publisher.models.external_api_property_schema import ExternalAPIPropertySchema
from typing import List


class ExtProviderDTO(BaseModel):
    name: Optional[str] = None
    properties: Optional['List[ExternalAPIPropertySchema]'] = None
    form_fields: Optional['List[ExternalAPIField]'] = None
    can_enforce_uniqueness: Optional[bool] = None
    enforce_uniqueness_by_default: Optional[bool] = None


ExtProviderDTO.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user import User
from pianosdk.publisher.models.webhook_response import WebhookResponse


class WebhookEvent(BaseModel):
    webhook_id: Optional[str] = None
    status: Optional[str] = None
    status_localized: Optional[str] = None
    retried: Optional[str] = None
    create_date: Optional[datetime] = None
    update_date: Optional[datetime] = None
    last_webhook_response: Optional['WebhookResponse'] = None
    user: Optional['User'] = None
    type: Optional[str] = None
    type_localized: Optional[str] = None
    event: Optional[str] = None
    event_localized: Optional[str] = None
    event_type: Optional[str] = None
    responses_count: Optional[int] = None


WebhookEvent.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ExperienceMetadata(BaseModel):
    aid: Optional[str] = None
    experience_id: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None
    status: Optional[str] = None
    hierarchy_type: Optional[str] = None
    parent_id: Optional[str] = None
    create_date: Optional[datetime] = None
    create_by: Optional[str] = None
    update_date: Optional[datetime] = None
    update_by: Optional[str] = None
    description: Optional[str] = None
    schedule: Optional[str] = None
    deleted: Optional[bool] = None
    major_version: Optional[int] = None
    minor_version: Optional[int] = None


ExperienceMetadata.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class ConfigurationPropertyMetadata(BaseModel):
    name: Optional[str] = None
    attributes: Optional[str] = None
    validation_rules: Optional[str] = None
    properties: Optional['List[ConfigurationPropertyMetadata]'] = None


ConfigurationPropertyMetadata.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user import User


class GlobalTemplateVersion(BaseModel):
    offer_template_id: Optional[str] = None
    token_type: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    aid: Optional[str] = None
    type: Optional[str] = None
    type_id: Optional[str] = None
    category_id: Optional[str] = None
    published: Optional[bool] = None
    publish_date: Optional[datetime] = None
    version: Optional[int] = None
    create_date: Optional[datetime] = None
    create_by: Optional['User'] = None
    update_date: Optional[datetime] = None
    update_by: Optional['User'] = None
    status: Optional[str] = None


GlobalTemplateVersion.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.access import Access
from pianosdk.publisher.models.access_dto import AccessDTO


class PublisherUserAccessApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def check_access(self, aid: str, uid: str, rid: str, cross_app: bool = False) -> ApiResponse[AccessDTO]:
        _url_path = '/api/v3/publisher/user/access/check'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'rid': _encode_parameter(rid),
            'cross_app': _encode_parameter(cross_app)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, AccessDTO)
        return _result

    def grant_access(self, aid: str, rid: str, send_email: bool = False, uid: str = None, emails: str = None, expire_date: int = None, url: str = None, message: str = None) -> ApiResponse[List[Access]]:
        _url_path = '/api/v3/publisher/user/access/grant'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid),
            'uid': _encode_parameter(uid),
            'emails': _encode_parameter(emails),
            'expire_date': _encode_parameter(expire_date),
            'send_email': _encode_parameter(send_email),
            'url': _encode_parameter(url),
            'message': _encode_parameter(message)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Access)
        return _result

    def grant_access_to_users(self, aid: str, rid: List[str], send_email: bool = False, emails: List[str] = None, file_id: str = None, expire_date: datetime = None, message: str = None) -> ApiResponse[int]:
        _url_path = '/api/v3/publisher/user/access/grantToUsers'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid),
            'emails': _encode_parameter(emails),
            'file_id': _encode_parameter(file_id),
            'expire_date': _encode_parameter(expire_date),
            'send_email': _encode_parameter(send_email),
            'message': _encode_parameter(message)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, int)
        return _result

    def list_access(self, aid: str, uid: str, offset: int = 0, limit: int = 100, expand_bundled: bool = False, cross_app: bool = False) -> ApiResponse[List[AccessDTO]]:
        _url_path = '/api/v3/publisher/user/access/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'uid': _encode_parameter(uid),
            'expand_bundled': _encode_parameter(expand_bundled),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'cross_app': _encode_parameter(cross_app)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, AccessDTO)
        return _result

    def revoke_access(self, access_id: str) -> ApiResponse[Access]:
        _url_path = '/api/v3/publisher/user/access/revoke'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'access_id': _encode_parameter(access_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Access)
        return _result

    def update(self, access_id: str, expire_date: datetime = None) -> ApiResponse[Access]:
        _url_path = '/api/v3/publisher/user/access/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'access_id': _encode_parameter(access_id),
            'expire_date': _encode_parameter(expire_date)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Access)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class EarlyBillingDetails(BaseModel):
    enabled: Optional[bool] = None
    in_progress: Optional[bool] = None
    billing_date: Optional[datetime] = None


EarlyBillingDetails.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class CheckCanUpgradeResult(BaseModel):
    email: Optional[str] = None
    error_message: Optional[str] = None
    error_code: Optional[int] = None
    is_success: Optional[bool] = None


CheckCanUpgradeResult.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.term_conversion import TermConversion


class PublisherConversionExternalApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def external_verification_create(self, aid: str, term_id: str, uid: str, fields: str, check_validity: bool = False, access_to: datetime = None, tracking_id: str = None, custom_params: str = None, browser_id: str = None, page_title: str = None, url: str = None, referer: str = None, content_author: str = None, content_created: str = None, content_section: str = None, content_type: str = None, tags: List[str] = None, previous_user_segments: str = None, user_state: str = None, cookie_consents: str = None) -> ApiResponse[TermConversion]:
        _url_path = '/api/v3/publisher/conversion/external/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'term_id': _encode_parameter(term_id),
            'uid': _encode_parameter(uid),
            'fields': _encode_parameter(fields),
            'check_validity': _encode_parameter(check_validity),
            'access_to': _encode_parameter(access_to),
            'tracking_id': _encode_parameter(tracking_id),
            'custom_params': _encode_parameter(custom_params),
            'browser_id': _encode_parameter(browser_id),
            'page_title': _encode_parameter(page_title),
            'url': _encode_parameter(url),
            'referer': _encode_parameter(referer),
            'content_author': _encode_parameter(content_author),
            'content_created': _encode_parameter(content_created),
            'content_section': _encode_parameter(content_section),
            'content_type': _encode_parameter(content_type),
            'tags': _encode_parameter(tags),
            'previous_user_segments': _encode_parameter(previous_user_segments),
            'user_state': _encode_parameter(user_state),
            'cookie_consents': _encode_parameter(cookie_consents)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, TermConversion)
        return _result




from copy import deepcopy
from typing import Dict
from urllib.parse import urlparse, urlunparse

from pianosdk.httpwrap import RequestsClient


class Configuration:

    @property
    def http_client(self) -> RequestsClient:
        """
        The Http Client, which makes all requests
        :return:
        """
        return self._http_client

    @property
    def timeout(self) -> int:
        """
        :return: The value used for connection timeout
        """
        return self._timeout

    @property
    def max_retries(self) -> int:
        """
        :return: The number of times to retry failed endpoint call
        """
        return self._max_retries

    @property
    def backoff_factor(self) -> int:
        """
        Used for sleep `{backoff factor} * (2 ** ({number of total retries} - 1))`
        :return: A backoff factor to apply between attempts after the second try.
        """
        return self._backoff_factor

    @property
    def environment(self) -> str:
        """
        :return: API environment: `production` or `sandbox`
        """
        return self._environment

    @property
    def api_token(self) -> str:
        """
        :return: Api Token
        """
        return self._api_token

    @property
    def private_key(self) -> str:
        """
        :return: Private key
        """
        return self._private_key

    @property
    def additional_headers(self) -> Dict:
        """
        :return: Copy of additional headers added to each API request
        """
        return deepcopy(self._additional_headers)

    def __init__(self, timeout: int = 60, max_retries: int = 3, backoff_factor: int = 0,
                 api_host: str = 'production', api_token: str = 'TODO: Replace', private_key: str = 'TODO: Replace',
                 additional_headers: Dict = {}):
        """Configuration object constructor
        :param timeout: The value to use for connection timeout
        :param max_retries: The number of times to retry failed endpoint call
        :param backoff_factor: A backoff factor to apply between attempts after the second try.
        :param api_host: API host. Use `production`, `sandbox` or custom url
        :param api_token: Api Token
        :param private_key: Private key
        :param additional_headers: Additional headers to add to each API request
        """
        self._host = Configuration._environments.get(api_host, None) or Configuration._validate_host(api_host)
        self._timeout = timeout
        self._max_retries = max_retries
        self._backoff_factor = backoff_factor
        self._api_token = api_token
        self._private_key = private_key
        self._additional_headers = deepcopy(additional_headers) or {}
        self._http_client = self.create_http_client()

    _environments: Dict[str, str] = {
        'production': "https://api.piano.io",
        'sandbox': "https://sandbox.piano.io"
    }

    @staticmethod
    def _validate_host(url):
        url_data = urlparse(url)
        if url_data.scheme and url_data.netloc:
            return urlunparse(list(url_data)[:2] + [''] * 4)
        raise ValueError(f'Invalid API host {url}.')

    def get_base_url(self) -> str:
        return self._host

    def create_http_client(self) -> RequestsClient:
        return RequestsClient(timeout=self.timeout,
                              max_retries=self.max_retries,
                              backoff_factor=self.backoff_factor)



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.price_dto import PriceDTO
from pianosdk.publisher.models.user_alias_dto import UserAliasDto
from typing import List


class UserDetails(BaseModel):
    uid: Optional[str] = None
    name: Optional[str] = None
    personal_name: Optional[str] = None
    display_name: Optional[str] = None
    image1: Optional[str] = None
    access_count: Optional[int] = None
    spent_money: Optional['List[PriceDTO]'] = None
    create_date: Optional[datetime] = None
    email: Optional[str] = None
    email_state: Optional[int] = None
    last_active_date: Optional[datetime] = None
    user_aliases: Optional['List[UserAliasDto]'] = None


UserDetails.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class VoucheringPolicy(BaseModel):
    vouchering_policy_id: Optional[str] = None
    vouchering_policy_billing_plan: Optional[str] = None
    vouchering_policy_billing_plan_description: Optional[str] = None
    vouchering_policy_redemption_url: Optional[str] = None


VoucheringPolicy.model_rebuild()



from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='pianosdk',
    version='16.526.0',
    packages=find_packages(),
    # url='url will be here',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries',
        
    ],
    license='Apache License, Version 2.0',
    description='Piano API SDK',
    long_description=readme,
    long_description_content_type='text/markdown',
    python_requires='>=3.7',
    install_requires=[
        'requests>=2.28.1',
        'pydantic==2.5.3',
        'pycryptodome>=3.15.0'
    ]
)



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class OAuthToken(BaseModel):
    access_token: Optional[str] = None
    expires_in: Optional[int] = None
    refresh_token: Optional[str] = None
    token_type: Optional[str] = None


OAuthToken.model_rebuild()



from pianosdk.access_token import AccessTokenStorage, AccessToken
from pianosdk.api_response import ApiResponse
from pianosdk.configuration import Configuration
from pianosdk.constants import __version__, __license__, __copyright__
from pianosdk.exceptions import ApiException
from pianosdk.httpwrap import HttpCallBack
from pianosdk.piano_client import Client
from pianosdk.webhook_events import UnknownEvent, TestEvent, UserPaymentMethodEvent, UnsupportedEvent, PaymentRefundEvent, AddressUpdatedEvent, TermChangeEvent, SubscriptionCreatedEvent, KeyingEvent, TermChangedEvent, SubscriptionAutoRenewChangedByEndUserEvent, UserAddressUpdatedEvent, SharedAccessGrantedEvent, VoucherRedeemedEvent, VoucherDeliveredEvent, VoucherRevokedEvent, VoucherPurchasedEvent, PaymentPendingEvent, PaymentFailedEvent, PaymentInitiatedEvent, PaymentCompletedEvent, UserUpdatedEvent, UserEmailConfirmedEvent, UserCreatedEvent, UserDisabledEvent, ContractUpdatedEvent, ContractCreatedEvent, ContractUserCreatedEvent, ContractUserInvitedEvent, ContractDeletedEvent, SubscriptionEndedEvent, SubscriptionRenewalEvent, SubscriptionUpdatedEvent, AccessGrantedEvent, AccessRevokedEvent, AccessModifiedEvent, LicenseeCreatedEvent, LicenseeUpdatedEvent, TermChangeFinishedEvent, ContractUserAccessExpiredEvent, ContractUserAccessRevokedEvent, ContractRedeemedEvent, ContractRenewedEvent, SharedSubscriptionChildAccessGrantedEvent



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.offer_template_content_field import OfferTemplateContentField
from pianosdk.publisher.models.offer_template_variant import OfferTemplateVariant
from pianosdk.publisher.models.user import User
from typing import List


class OfferTemplate(BaseModel):
    offer_template_id: Optional[str] = None
    token_type: Optional[str] = None
    aid: Optional[str] = None
    app_name: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    deleted: Optional[bool] = None
    create_date: Optional[datetime] = None
    create_by: Optional['User'] = None
    update_date: Optional[datetime] = None
    update_by: Optional['User'] = None
    publish_date: Optional[datetime] = None
    version: Optional[int] = None
    type: Optional[str] = None
    type_id: Optional[str] = None
    boilerplate_type: Optional[str] = None
    boilerplate_type_id: Optional[str] = None
    category_id: Optional[str] = None
    thumbnail_image_url: Optional[str] = None
    live_thumbnail_image_url: Optional[str] = None
    status: Optional[str] = None
    is_published: Optional[bool] = None
    count_variants: Optional[int] = None
    variant_list: Optional['List[OfferTemplateVariant]'] = None
    archived_date: Optional[datetime] = None
    archived_by: Optional['User'] = None
    content_field_list: Optional['List[OfferTemplateContentField]'] = None
    can_be_global: Optional[bool] = None
    is_global: Optional[bool] = None
    is_inherited: Optional[bool] = None


OfferTemplate.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ResourceTag(BaseModel):
    resource_tag_id: Optional[str] = None
    name: Optional[str] = None
    type: Optional[str] = None


ResourceTag.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.delivery_period_read_view import DeliveryPeriodReadView
from typing import List


class DeliveryScheduleReadView(BaseModel):
    delivery_periods: Optional['List[DeliveryPeriodReadView]'] = None
    edit_threshold: Optional[str] = None
    main_period_pub_id: Optional[str] = None


DeliveryScheduleReadView.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.licensee import Licensee


class PublisherLicensingLicenseeApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def archive_licensee(self, aid: str, licensee_id: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/licensing/licensee/archive'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def create_licensee(self, aid: str, manager_uids: List[str], name: str, description: str = None, representatives: str = None, logo_url: str = None) -> ApiResponse[Licensee]:
        _url_path = '/api/v3/publisher/licensing/licensee/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'description': _encode_parameter(description),
            'manager_uids': _encode_parameter(manager_uids),
            'representatives': _encode_parameter(representatives),
            'logo_url': _encode_parameter(logo_url),
            'name': _encode_parameter(name)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Licensee)
        return _result

    def get_licensee(self, aid: str, licensee_id: str) -> ApiResponse[Licensee]:
        _url_path = '/api/v3/publisher/licensing/licensee/get'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Licensee)
        return _result

    def get_licensee_count(self, aid: str) -> ApiResponse[int]:
        _url_path = '/api/v3/publisher/licensing/licensee/count'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, int)
        return _result

    def get_licensee_list(self, aid: str, offset: int = 0, limit: int = 100, q: str = None) -> ApiResponse[List[Licensee]]:
        _url_path = '/api/v3/publisher/licensing/licensee/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Licensee)
        return _result

    def update_licensee(self, aid: str, licensee_id: str, manager_uids: List[str], name: str, description: str = None, representatives: str = None, logo_url: str = None) -> ApiResponse[Licensee]:
        _url_path = '/api/v3/publisher/licensing/licensee/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'licensee_id': _encode_parameter(licensee_id),
            'description': _encode_parameter(description),
            'manager_uids': _encode_parameter(manager_uids),
            'representatives': _encode_parameter(representatives),
            'logo_url': _encode_parameter(logo_url),
            'name': _encode_parameter(name)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Licensee)
        return _result

